package com.javacodegeeks.nio.iterate_directory;

import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.DirectoryStream.Filter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Objects;
import java.util.Optional;

import com.javacodegeeks.nio.iterate_directory.runner.CLIConfig;

public final class Listing {

    private final CLIConfig config;
    private final ConsoleView view;

    public Listing(final CLIConfig config) {
        this.config = config;
        this.view = new ConsoleView(config);
    }

    public void list() {        
        final Path root = Paths.get(this.config.getRoot().orElseThrow(RuntimeException::new));

        if (Files.isDirectory(root)) {
            final Directory rootDir = new Directory(root);

            list(rootDir, this.config.getFilter());
            this.view.render(rootDir);
        } else {
            throw new RuntimeException("Root needs to be a directory");
        }
    }

    private void list(final Directory directory, final Optional<String> filter) {
        assert !Objects.isNull(directory) && !Objects.isNull(filter);

        DirectoryStream<Path> stream = null;
        try {
            if (filter.isPresent()) {
                stream = Files.newDirectoryStream(directory.getPath(), createFilter(filter.get()));
            } else {
                stream = Files.newDirectoryStream(directory.getPath());
            }

            for (Path path : stream) {
                if (Files.isDirectory(path)) {
                    final Directory child = new Directory(path);
                    directory.addDirectory(child);

                    list(child, filter);
                } else {
                    directory.addFile(new File(path));
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if (!Objects.isNull(stream)) {
                try {
                    stream.close();
                } catch (IOException e) {
                    throw new RuntimeException("unable to close stream while listing directories", e);
                }
            }
        }
    }

    private Filter<Path> createFilter(final String pattern) {

        return new Filter<Path>() {

            @Override
            public boolean accept(final Path entry) throws IOException {
                return Files.isDirectory(entry) || entry.toFile().getName().contains(pattern);
            }
        };
    }
}
